@@ -53,7 +53,6 @@
 #include "proc/sig.h"
 #include "proc/devname.h"
 #include "proc/sysinfo.h"
-#include "proc/version.h" /* procps_version */
 
 static int i_am_pkill = 0;
 
@@ -118,6 +117,7 @@ static int __attribute__ ((__noreturn__)) usage(int opt)
 	fputs(_(" -f, --full                use full process name to match\n"), fp);
 	fputs(_(" -g, --pgroup <PGID,...>   match listed process group IDs\n"), fp);
 	fputs(_(" -G, --group <GID,...>     match real group IDs\n"), fp);
+	fputs(_(" -i, --ignore-case         match case insensitively\n"), fp);
 	fputs(_(" -n, --newest              select most recently started\n"), fp);
 	fputs(_(" -o, --oldest              select least recently started\n"), fp);
 	fputs(_(" -P, --parent <PPID,...>   match only child processes of the given parent\n"), fp);
@@ -160,7 +160,7 @@ static struct el *split_list (const char *restrict str, int (*convert)(const cha
 		if (i == size) {
 			size = size * 5 / 4 + 4;
 			/* add 1 because slot zero is a count */
-			list = xrealloc (list, 1 + size * sizeof *list);
+			list = xrealloc (list, (1 + size) * sizeof *list);
 		}
 		sep_pos = strchr (ptr, ',');
 		if (sep_pos)
@@ -629,7 +629,7 @@ static struct el * select_procs (int *num)
 						if (list == NULL)
 							exit (EXIT_FATAL);
 					}
-					if (opt_long) {
+					if (opt_long || opt_longlong) {
 						list[matches].str = xstrdup (cmdoutput);
 						list[matches++].num = subtask.XXXID;
 					} else {
@@ -661,8 +661,6 @@ static int signal_option(int *argc, char **argv)
 	for (i = 1; i < *argc; i++) {
 		if (argv[i][0] == '-') {
 			sig = signal_name_to_number(argv[i] + 1);
-			if (sig == -1 && isdigit(argv[i][1]))
-				sig = atoi(argv[i] + 1);
 			if (-1 < sig) {
 				memmove(argv + i, argv + i + 1,
 					sizeof(char *) * (*argc - i));
@@ -676,7 +674,7 @@ static int signal_option(int *argc, char **argv)
 
 static void parse_opts (int argc, char **argv)
 {
-	char opts[32] = "";
+	char opts[64] = "";
 	int opt;
 	int criteria_count = 0;
 
@@ -694,6 +692,7 @@ static void parse_opts (int argc, char **argv)
 		{"full", no_argument, NULL, 'f'},
 		{"pgroup", required_argument, NULL, 'g'},
 		{"group", required_argument, NULL, 'G'},
+		{"ignore-case", no_argument, NULL, 'i'},
 		{"newest", no_argument, NULL, 'n'},
 		{"oldest", no_argument, NULL, 'o'},
 		{"parent", required_argument, NULL, 'P'},
@@ -727,7 +726,7 @@ static void parse_opts (int argc, char **argv)
 		strcat (opts, "lad:vw");
 	}
 
-	strcat (opts, "LF:cfnoxP:g:s:u:U:G:t:?Vh");
+	strcat (opts, "LF:cfinoxP:g:s:u:U:G:t:?Vh");
 
 	while ((opt = getopt_long (argc, argv, opts, longopts, NULL)) != -1) {
 		switch (opt) {
@@ -798,11 +797,11 @@ static void parse_opts (int argc, char **argv)
 				usage ('?');
 			++criteria_count;
 			break;
-/*		case 'i':   / * FreeBSD: ignore case. OpenBSD: withdrawn. See -I. This sucks. * /
- *			if (opt_case)
- *				usage (opt);
- *			opt_case = REG_ICASE;
- *			break; */
+		case 'i':   /* FreeBSD: ignore case. OpenBSD: withdrawn. See -I. This sucks. */
+			if (opt_case)
+				usage (opt);
+			opt_case = REG_ICASE;
+			break;
 /*		case 'j':   / * FreeBSD: restricted to the given jail ID * /
  *			break; */
 		case 'l':   /* Solaris: long output format (pgrep only) Should require -f for beyond argv[0] maybe? */
