@@ -1,29 +1,36 @@
 /*
- * Copyright 1998-2004 by Albert Cahalan; all rights resered.         
- * This file may be used subject to the terms and conditions of the
- * GNU Library General Public License Version 2, or any later version  
- * at your option, as published by the Free Software Foundation.
- * This program is distributed in the hope that it will be useful,
+ * sortformat - ps output sorting
+ * Copyright 1998-2004 by Albert Cahalan
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- * GNU Library General Public License for more details.
- */                                 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ */
 
-#include <stdlib.h>
+#include <grp.h>
+#include <pwd.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 
-/* username lookups */
 #include <sys/types.h>
-#include <pwd.h>
-#include <grp.h>
 
 #include "../proc/readproc.h"
 #include "../proc/sysinfo.h"
+
 #include "common.h"
 
 static sf_node *sf_list = NULL;         /* deferred sorting and formatting */
-static int broken;                      /* use gross Unix98 parsing? */
 static int have_gnu_sort = 0;           /* if true, "O" must be format */
 static int already_parsed_sort = 0;     /* redundantly set in & out of fn */
 static int already_parsed_format = 0;
@@ -49,12 +56,10 @@ static format_node *do_one_spec(const char *spec, const char *override){
     if(override){
       w2 = strlen(override);
       thisnode->width = (w1>w2)?w1:w2;
-      thisnode->name = malloc(strlen(override)+1);
-      strcpy(thisnode->name, override);
+      thisnode->name = strdup(override);
     }else{
       thisnode->width = w1;
-      thisnode->name = malloc(strlen(fs->head)+1);
-      strcpy(thisnode->name, fs->head);
+      thisnode->name = strdup(fs->head);
     }
     thisnode->pr = fs->pr;
     thisnode->need = fs->need;
@@ -98,12 +103,12 @@ static void O_wrap(sf_node *sfn, int otype){
   trailer = (otype=='b') ? "END_BSD" : "END_SYS5" ;
 
   fnode =  do_one_spec("pid",NULL);
-  if(!fnode)fprintf(stderr,"Seriously crashing. Goodbye cruel world.\n");
+  if(!fnode)catastrophic_failure(__FILE__, __LINE__, _("seriously crashing: goodbye cruel world"));
   endp = sfn->f_cooked; while(endp->next) endp = endp->next;  /* find end */
   endp->next = fnode;
-  
+
   fnode =  do_one_spec(trailer,NULL);
-  if(!fnode)fprintf(stderr,"Seriously crashing. Goodbye cruel world.\n");
+  if(!fnode)catastrophic_failure(__FILE__, __LINE__, _("seriously crashing: goodbye cruel world"));
   endp = fnode; while(endp->next) endp = endp->next;  /* find end */
   endp->next = sfn->f_cooked;
   sfn->f_cooked = fnode;
@@ -138,16 +143,15 @@ static const char *aix_format_parse(sf_node *sfn){
     items++;
     c = *walk++;
     if(c)         goto initial;
-    return "Improper AIX field descriptor.";
+    return _("improper AIX field descriptor");
   looks_ok:
     ;
   }
 
   /*** sanity check passed ***/
-  buf = malloc(strlen(sfn->sf)+1);
-  strcpy(buf, sfn->sf);
+  buf = strdup(sfn->sf);
   walk = sfn->sf;
-  
+
   while(items--){
     format_node *fnode;  /* newly allocated */
     format_node *endp;   /* for list manipulation */
@@ -160,12 +164,12 @@ static const char *aix_format_parse(sf_node *sfn){
       walk++;
       if(!aix){
         free(buf);
-        return "Unknown AIX field descriptor.";
+        return _("unknown AIX field descriptor");
       }
       fnode =  do_one_spec(aix->spec, aix->head);
       if(!fnode){
         free(buf);
-        return "AIX field descriptor processing bug.";
+        return _("AIX field descriptor processing bug");
       }
     } else {
       int len;
@@ -180,15 +184,14 @@ double_percent:
       walk += len;
       fnode = malloc(sizeof(format_node));
       fnode->width = len;
-      fnode->name = malloc(len+1);
-      strcpy(fnode->name, buf);
+      fnode->name = strdup(buf);
       fnode->pr = NULL;     /* checked for */
       fnode->need = 0;
       fnode->vendor = AIX;
       fnode->flags = CF_PRINT_EVERY_TIME;
       fnode->next = NULL;
     }
-    
+
     endp = fnode; while(endp->next) endp = endp->next;  /* find end */
     endp->next = sfn->f_cooked;
     sfn->f_cooked = fnode;
@@ -201,7 +204,6 @@ double_percent:
 /***************************************************************
  * Used to parse option O lists. Option O is shared between
  * sorting and formatting. Users may expect one or the other.
- * The "broken" flag enables a really bad Unix98 misfeature.
  * Put each completed format_node onto the list starting at ->f_cooked
  */
 static const char *format_parse(sf_node *sfn){
@@ -215,9 +217,8 @@ static const char *format_parse(sf_node *sfn){
   static char errbuf[80]; /* for variable-text error message */
 
   /*** prepare to operate ***/
-  buf = malloc(strlen(sfn->sf)+1);
-  strcpy(buf, sfn->sf);
-  
+  buf = strdup(sfn->sf);
+
   /*** sanity check and count items ***/
   need_item = 1; /* true */
   items = 0;
@@ -232,9 +233,6 @@ static const char *format_parse(sf_node *sfn){
       }
       need_item=1;
       break;
-    case '=':
-      if(broken) goto out;
-      /* fall through */
     default:
       if(need_item) items++;
       need_item=0;
@@ -284,7 +282,7 @@ out:
         snprintf(
           errbuf,
           sizeof(errbuf),
-          "Unknown user-defined format specifier \"%s\".",
+          _("unknown user-defined format specifier \"%s\""),
           walk
         );
       }
@@ -310,10 +308,10 @@ out:
 
   /* errors may cause a retry looking for AIX format codes */
   if(0) unknown:  err=errbuf;
-  if(0) empty:    err="Empty format list.";
-  if(0) improper: err="Improper format list.";
-  if(0) badwidth: err="Column widths must be unsigned decimal numbers.";
-  if(0) notmacro: err="Can't set width for a macro (multi-column) format specifier.";
+  if(0) empty:    err=_("empty format list");
+  if(0) improper: err=_("improper format list");
+  if(0) badwidth: err=_("column widths must be unsigned decimal numbers");
+  if(0) notmacro: err=_("can not set width for a macro (multi-column) format specifier");
   if(strchr(sfn->sf,'%')) err = aix_format_parse(sfn);
   return err;
 }
@@ -355,9 +353,8 @@ static const char *long_sort_parse(sf_node *sfn){
   int need_item;
 
   /*** prepare to operate ***/
-  buf = malloc(strlen(sfn->sf)+1);
-  strcpy(buf, sfn->sf);
-  
+  buf = strdup(sfn->sf);
+
   /*** sanity check and count items ***/
   need_item = 1; /* true */
   items = 0;
@@ -367,7 +364,7 @@ static const char *long_sort_parse(sf_node *sfn){
     case ' ': case ',': case '\t': case '\n': case '\0':
       if(need_item){
         free(buf);
-        return "Improper sort list";
+        return _("improper sort list");
       }
       need_item=1;
       break;
@@ -378,12 +375,12 @@ static const char *long_sort_parse(sf_node *sfn){
   } while (*++walk);
   if(!items){
     free(buf);
-    return "Empty sort list.";
+    return _("empty sort list");
   }
 #ifdef STRICT_LIST
   if(need_item){    /* can't have trailing deliminator */
     free(buf);
-    return "Improper sort list.";
+    return _("improper sort list");
   }
 #else
   if(need_item){    /* allow 1 trailing deliminator */
@@ -399,7 +396,7 @@ static const char *long_sort_parse(sf_node *sfn){
     snode = do_one_sort_spec(walk);
     if(!snode){
       free(buf);
-      return "Unknown sort specifier.";
+      return _("unknown sort specifier");
     }
     endp = snode; while(endp->next) endp = endp->next;  /* find end */
     endp->next = sfn->s_cooked;
@@ -426,7 +423,7 @@ static const char *verify_short_sort(const char *arg){
   int i;
   const char *walk;
   int tmp;
-  if(strspn(arg,all) != strlen(arg)) return "Bad sorting code.";
+  if(strspn(arg,all) != strlen(arg)) return _("bad sorting code");
   for(i=256; i--;) checkoff[i] = 0;
   walk = arg;
   for(;;){
@@ -437,13 +434,13 @@ static const char *verify_short_sort(const char *arg){
     case '+':
     case '-':
       tmp = *(walk+1);
-      if(!tmp || tmp=='+' || tmp=='-') return "Bad sorting code.";
+      if(!tmp || tmp=='+' || tmp=='-') return _("bad sorting code");
       break;
     case 'P':
-      if(forest_type) return "PPID sort and forest output conflict.";
+      if(forest_type) return _("PPID sort and forest output conflict");
       /* fall through */
     default:
-      if(checkoff[tmp]) return "Bad sorting code.";   /* repeated */
+      if(checkoff[tmp]) return _("bad sorting code");   /* repeated */
       /* ought to check against already accepted sort options */
       checkoff[tmp] = 1;
       break;
@@ -477,9 +474,9 @@ static const char *short_sort_parse(sf_node *sfn){
       break;
     default:
       ss = search_shortsort_array(tmp);
-      if(!ss) return "Unknown sort specifier.";
+      if(!ss) return _("unknown sort specifier");
       snode = do_one_sort_spec(ss->spec);
-      if(!snode) return "Unknown sort specifier.";
+      if(!snode) return _("unknown sort specifier");
       snode->reverse = direction;
       endp = snode; while(endp->next) endp = endp->next;  /* find end */
       endp->next = sfn->s_cooked;
@@ -497,7 +494,6 @@ static const char *short_sort_parse(sf_node *sfn){
 /*
  * Used to parse option O lists. Option O is shared between
  * sorting and formatting. Users may expect one or the other.
- * The "broken" flag enables a really bad Unix98 misfeature.
  * Recursion is to preserve original order.
  */
 static const char *parse_O_option(sf_node *sfn){
@@ -515,14 +511,14 @@ static const char *parse_O_option(sf_node *sfn){
       break;
     case SF_U_O:                                /*** format ***/
       /* Can have -l -f f u... set already_parsed_format like DEC does */
-      if(already_parsed_format) return "option -O can not follow other format options.";
+      if(already_parsed_format) return _("option -O can not follow other format options");
       err = format_parse(sfn);
       if(err) return err;
       already_parsed_format = 1;
       O_wrap(sfn,'u'); /* must wrap user format in default */
       break;
     case SF_B_O:                                /***  both  ***/
-      if(have_gnu_sort || already_parsed_sort) err = "Multiple sort options.";
+      if(have_gnu_sort || already_parsed_sort) err = _("multiple sort options");
       else err = verify_short_sort(sfn->sf);
       if(!err){ /* success as sorting code */
         short_sort_parse(sfn);
@@ -530,7 +526,7 @@ static const char *parse_O_option(sf_node *sfn){
         return NULL;
       }
       if(already_parsed_format){
-        err = "option O is neither first format nor sort order.";
+        err = _("option O is neither first format nor sort order");
         break;
       }
       if(!format_parse(sfn)){ /* if success as format code */
@@ -540,12 +536,12 @@ static const char *parse_O_option(sf_node *sfn){
       }
       break;
     case SF_G_sort: case SF_B_m:                 /***  sort  ***/
-      if(already_parsed_sort) err = "Multiple sort options.";
+      if(already_parsed_sort) err = _("multiple sort options");
       else err = long_sort_parse(sfn);
       already_parsed_sort = 1;
       break;
     default:                                    /***  junk  ***/
-      return "Bug: parse_O_option got weirdness!";
+      catastrophic_failure(__FILE__, __LINE__, _("please report this bug"));
   }
   return err; /* could be NULL */
 }
@@ -561,8 +557,7 @@ int defer_sf_option(const char *arg, int source){
   int need_item = 1;
 
   sfn = malloc(sizeof(sf_node));
-  sfn->sf = malloc(strlen(arg)+1);
-  strcpy(sfn->sf, arg);
+  sfn->sf = strdup(arg);
   sfn->sf_code = source;
   sfn->s_cooked = NULL;
   sfn->f_cooked = NULL;
@@ -658,7 +653,7 @@ static int fmt_delete(const char *findme){
 static const char *generate_sysv_list(void){
   format_node *fn;
   if((format_modifiers & FM_y) && !(format_flags & FF_Ul))
-    return "Modifier -y without format -l makes no sense.";
+    return _("modifier -y without format -l makes no sense");
   if(prefer_bsd_defaults){
     if(format_flags) PUSH("cmd");
     else PUSH("args");
@@ -687,8 +682,7 @@ static const char *generate_sysv_list(void){
       PUSH("sgi_rss");
       fn = malloc(sizeof(format_node));
       fn->width = 1;
-      fn->name = malloc(2);
-      strcpy(fn->name, ":");
+      fn->name = strdup(":");
       fn->pr = NULL;     /* checked for */
       fn->need = 0;
       fn->vendor = AIX;   /* yes, for SGI weirdness */
@@ -750,19 +744,16 @@ static const char *generate_sysv_list(void){
  * sorting and formatting. Users may expect one or the other.
  * The "broken" flag enables a really bad Unix98 misfeature.
  */
-const char *process_sf_options(int localbroken){
+const char *process_sf_options(void){
   sf_node *sf_walk;
 
-  if(personality & PER_BROKEN_o) localbroken = 1;
-  if(personality & PER_GOOD_o)   localbroken = 0;
-  broken = localbroken;
   if(sf_list){
     const char *err;
     err = parse_O_option(sf_list);
     if(err) return err;
   }
 
-  if(format_list) printf("Bug: must reset the list first!\n");
+  if(format_list) catastrophic_failure(__FILE__, __LINE__, _("bug: must reset the list first"));
 
   /* merge formatting info of sf_list into format_list here */
   sf_walk = sf_list;
@@ -800,7 +791,7 @@ const char *process_sf_options(int localbroken){
   // with sorting. Do the threads remain grouped, with sorting
   // by process, or do the threads get sorted by themselves?
   if(sort_list && (thread_flags&TF_no_sort)){
-    return "Tell procps-feedback@lists.sf.net what you expected.";
+    return _("tell <procps@freelists.org> what you expected");
   }
 
   // If nothing else, try to use $PS_FORMAT before the default.
@@ -810,7 +801,7 @@ const char *process_sf_options(int localbroken){
     if(tmp && *tmp){
       const char *err;
       sf_node sfn;
-      if(thread_flags&TF_must_use) return "Tell procps-feedback@sf.net what you want. (-L/-T, -m/m/H, and $PS_FORMAT)";
+      if(thread_flags&TF_must_use) return _("tell <procps@freelists.org> what you want (-L/-T, -m/m/H, and $PS_FORMAT)");
       sfn.sf = tmp;
       sfn.f_cooked = NULL;
       err = format_parse(&sfn);
@@ -827,14 +818,14 @@ const char *process_sf_options(int localbroken){
         return NULL;
       }
       // FIXME: prove that this won't be hit on valid bogus-BSD options
-      fprintf(stderr, "Warning: $PS_FORMAT ignored. (%s)\n", err);
+      fprintf(stderr, _("warning: $PS_FORMAT ignored. (%s)\n"), err);
     }
   }
 
   if(format_list){
-    if(format_flags) return "Conflicting format options.";
-    if(format_modifiers) return "Can't use output modifiers with user-defined output";
-    if(thread_flags&TF_must_use) return "-L/-T with H/m/-m and -o/-O/o/O is nonsense";
+    if(format_flags) return _("conflicting format options");
+    if(format_modifiers) return _("can not use output modifiers with user-defined output");
+    if(thread_flags&TF_must_use) return _("-L/-T with H/m/-m and -o/-O/o/O is nonsense");
     return NULL;
   }
 
@@ -842,7 +833,7 @@ const char *process_sf_options(int localbroken){
     const char *spec;
     switch(format_flags){
 
-    default:             return "Conflicting format options.";
+    default:             return _("conflicting format options");
 
     /* These can be NULL, which enables SysV list generation code. */
     case 0:              spec=NULL;           break;
@@ -892,9 +883,9 @@ const char *process_sf_options(int localbroken){
     if(format_modifiers & FM_j){
       fn = do_one_spec("pgid", NULL);
       if(!fmt_add_after("PPID", fn)) if(!fmt_add_after("PID", fn))
-        return "Internal error, no PID or PPID for -j option.";
+        catastrophic_failure(__FILE__, __LINE__, _("internal error: no PID or PPID for -j option"));
       fn = do_one_spec("sid", NULL);
-      if(!fmt_add_after("PGID", fn)) return "Lost my PGID!";
+      if(!fmt_add_after("PGID", fn)) return _("lost my PGID");
     }
     if(format_modifiers & FM_y){
       /* TODO: check for failure to do something, and complain if so */
@@ -907,15 +898,15 @@ const char *process_sf_options(int localbroken){
       fmt_delete("NI");
       fn = do_one_spec("class", NULL);
       if(!fmt_add_after("PRI", fn))
-        return "Internal error, no PRI for -c option.";
+        catastrophic_failure(__FILE__, __LINE__, _("internal error: no PRI for -c option"));
       fmt_delete("PRI"); /* we want a different one */
       fn = do_one_spec("pri", NULL);
-      if(!fmt_add_after("CLS", fn)) return "Lost my CLS!";
+      if(!fmt_add_after("CLS", fn)) return _("lost my CLS");
     }
     if(thread_flags & TF_U_T){
       fn = do_one_spec("spid", NULL);
       if(!fmt_add_after("PID", fn) && (thread_flags&TF_must_use))
-        return "-T with H/-m/m but no PID for SPID to follow";
+        return _("-T with H/-m/m but no PID for SPID to follow");
     }
     if(thread_flags & TF_U_L){
       fn = do_one_spec("lwp", NULL);
@@ -926,7 +917,7 @@ const char *process_sf_options(int localbroken){
       if(fmt_add_after("PPID", fn)) goto did_lwp;
       if(fmt_add_after("PID",  fn)) goto did_lwp;
       if(thread_flags&TF_must_use)
-        return "-L with H/-m/m but no PID/PGID/SID/SESS for NLWP to follow";
+        return _("-L with H/-m/m but no PID/PGID/SID/SESS for NLWP to follow");
 did_lwp:
       fn = do_one_spec("nlwp", NULL);
       fmt_add_after("%CPU",  fn);
