@@ -1,6 +1,6 @@
 /* top.h - Header file:         show Linux processes */
 /*
- * Copyright (c) 2002-2014, by: James C. Warner
+ * Copyright (c) 2002-2015, by: James C. Warner
  *    All rights reserved.      8921 Hilloway Road
  *                              Eden Prairie, Minnesota 55347 USA
  *
@@ -26,7 +26,6 @@
 //#define BOOST_PERCNT            /* enable extra precision for two % fields */
 //#define NOBOOST_MEMS            /* disable extra precision for mem fields  */
 //#define NUMA_DISABLE            /* disable summary area NUMA/Nodes display */
-//#define OOMEM_ENABLE            /* enable the SuSE out-of-memory additions */
 //#define ORIG_TOPDEFS            /* with no rcfile retain original defaults */
 //#define SIGNALS_LESS            /* favor reduced signal load over response */
 
@@ -59,6 +58,7 @@
 //#define STRINGCASENO            /* case insenstive compare/locate versions */
 //#define TERMIOS_ONLY            /* just limp along with native input only  */
 //#define TREE_NORESET            /* sort keys do NOT force forest view OFF  */
+//#define TREE_SCANALL            /* rescan array w/ forest view, avoid sort */
 //#define USE_X_COLHDR            /* emphasize header vs. whole col, for 'x' */
 //#define VALIDATE_NLS            /* validate the integrity of all nls tbls  */
 
@@ -197,13 +197,14 @@ enum pflag {
    EU_FL1, EU_FL2, EU_DRT,
    EU_STA, EU_CMD, EU_WCH, EU_FLG, EU_CGR,
    EU_SGD, EU_SGN, EU_TGD,
-#ifdef OOMEM_ENABLE
    EU_OOA, EU_OOM,
-#endif
    EU_ENV,
    EU_FV1, EU_FV2,
    EU_USE,
    EU_NS1, EU_NS2, EU_NS3, EU_NS4, EU_NS5, EU_NS6,
+   EU_LXC,
+   EU_RZA, EU_RZF, EU_RZL, EU_RZS,
+   EU_CGN,
 #ifdef USE_X_COLHDR
    // not really pflags, used with tbl indexing
    EU_MAXPFLGS
@@ -253,11 +254,11 @@ typedef struct HST_t {
    int pid;                     // record 'key'
 } HST_t;
 #else
-        /* This structure supports 'history' processing and records the
-           bare minimum of needed information from one frame to the next --
-           we don't calc and save data that goes unused like the old top nor
-           do we incure the overhead of sorting to support a binary search
-           (or worse, a friggin' for loop) when retrieval is necessary! */
+        /* This structure supports 'history' processing and records the bare
+           minimum of needed information from one frame to the next -- we do
+           not calc and save data that goes unused like the old top nor will
+           we incur the additional overhead of sort to support binary search
+           (or worse, a friggin' for loop) when retrievals become necessary! */
 typedef struct HST_t {
    TIC_t tics;                  // last frame's tics count
    unsigned long maj, min;      // last frame's maj/min_flt counts
@@ -605,16 +606,11 @@ typedef struct WIN_t {
 #define JOB_FIELDS  "(Ļ@<)*+,-./012568>?ABCFGHIJKLMNOPQRSTUVWXYZ[" RCF_PLUS_H
 #define MEM_FIELDS  "<MBND34&'()*+,-./0125689FGHIJKLOPQRSTUVWXYZ[" RCF_PLUS_H
 #define USR_FIELDS  ")+,-./1234568;<=>?@ABCFGHIJKLMNOPQRSTUVWXYZ[" RCF_PLUS_H
-#ifdef OOMEM_ENABLE
-        // the suse old top fields ( 'a'-'z' + '{|' ) in positions 0-27
-        // ( the extra chars above represent the 'off' state )
+        // old top fields ( 'a'-'z' ) in positions 0-25
+        // other suse old top fields ( '{|' ) in positions 26-27
 #define CVT_FIELDS  "%&*'(-0346789:;<=>?@ACDEFGML)+,./125BHIJKNOPQRSTUVWXYZ["
 #define CVT_FLDMAX  28
-#else
-        // other old top fields ( 'a'-'z' ) in positions 0-25
-#define CVT_FIELDS  "%&*'(-0346789:;<=>?@ACDEFG)+,./125BHIJKLMNOPQRSTUVWXYZ["
-#define CVT_FLDMAX  26
-#endif
+
 
         /* The default values for the local config file */
 #define DEF_RCFILE { \
@@ -680,7 +676,6 @@ typedef struct WIN_t {
 //atic void          at_eoj (void);
 //atic void          bye_bye (const char *str);
 //atic void          error_exit (const char *str);
-//atic void          library_err (const char *fmts, ...);
 //atic void          sig_abexit (int sig);
 //atic void          sig_endpgm (int dont_care_sig);
 //atic void          sig_paused (int dont_care_sig);
@@ -712,7 +707,7 @@ typedef struct WIN_t {
 /*------  Basic Formatting support  --------------------------------------*/
 //atic inline const char *justify_pad (const char *str, int width, int justr);
 //atic inline const char *make_chr (const char ch, int width, int justr);
-//atic inline const char *make_num (long num, int width, int justr, int col);
+//atic inline const char *make_num (long num, int width, int justr, int col, int noz);
 //atic inline const char *make_str (const char *str, int width, int justr, int col);
 //atic const char   *scale_mem (int target, unsigned long num, int width, int justr);
 //atic const char   *scale_num (unsigned long num, int width, int justr);
@@ -780,8 +775,10 @@ typedef struct WIN_t {
 //atic void          keys_window (int ch);
 //atic void          keys_xtra (int ch);
 /*------  Forest View support  -------------------------------------------*/
-//atic void          forest_adds (const int self, const int level);
+//atic void          forest_adds (const int self, int level);
+#ifndef TREE_SCANALL
 //atic int           forest_based (const proc_t **x, const proc_t **y);
+#endif
 //atic void          forest_create (WIN_t *q);
 //atic inline const char *forest_display (const WIN_t *q, const proc_t *p);
 /*------  Main Screen routines  ------------------------------------------*/
