@@ -49,6 +49,7 @@
 #include <time.h>
 #include <unistd.h>
 #ifdef WITH_WATCH8BIT
+# define _XOPEN_SOURCE_EXTENDED 1
 # include <wchar.h>
 # include <wctype.h>
 # include <ncursesw/ncurses.h>
@@ -135,7 +136,7 @@ static void init_ansi_colors(void)
 }
 
 
-static void set_ansi_attribute(const int attrib)
+static int set_ansi_attribute(const int attrib)
 {
 	switch (attrib) {
 	case -1:	/* restore last settings */
@@ -188,10 +189,12 @@ static void set_ansi_attribute(const int attrib)
 			fg_col = attrib - 30 + 1;
 		} else if (attrib >= 40 && attrib <= 47) { /* set background color */
 			bg_col = attrib - 40 + 1;
+		} else {
+			return 0; /* Not understood */
 		}
 	}
-
 	attrset(attributes | COLOR_PAIR(bg_col * nr_of_colors + fg_col + 1));
+    return 1;
 }
 
 static void process_ansi(FILE * fp)
@@ -212,9 +215,7 @@ static void process_ansi(FILE * fp)
 			buf[i] = '\0';
 			break;
 		}
-		if (c < '0' && c > '9' && c != ';') {
-			while (--i >= 0)
-				ungetc(buf[i], fp);
+		if ((c < '0' || c > '9') && c != ';') {
 			return;
 		}
 		buf[i] = (char)c;
@@ -226,8 +227,15 @@ static void process_ansi(FILE * fp)
 	 * escape sequence "<ESC>[0;1;31m". There can be 1 or more
 	 * attributes to apply, but typically there are between 1 and 3.
 	 */
-	for (endptr = numstart = buf; *endptr != '\0'; numstart = endptr + 1)
-		set_ansi_attribute(strtol(numstart, &endptr, 10));
+
+    /* Special case of <ESC>[m */
+    if (buf[0] == '\0')
+        set_ansi_attribute(0);
+
+	for (endptr = numstart = buf; *endptr != '\0'; numstart = endptr + 1) {
+		if (!set_ansi_attribute(strtol(numstart, &endptr, 10)))
+            break;
+    }
 }
 
 static void __attribute__ ((__noreturn__)) do_exit(int status)
@@ -355,66 +363,81 @@ wint_t my_getwc(FILE * s)
 #endif	/* WITH_WATCH8BIT */
 
 #ifdef WITH_WATCH8BIT
-static void output_header(wchar_t *restrict wcommand, int wcommand_columns, int wcommand_characters, double interval)
+static void output_header(wchar_t *restrict wcommand, int wcommand_characters, double interval)
 #else
 static void output_header(char *restrict command, double interval)
 #endif	/* WITH_WATCH8BIT */
 {
 	time_t t = time(NULL);
 	char *ts = ctime(&t);
-	int tsl = strlen(ts);
 	char *header;
+	char *right_header;
+	char hostname[HOST_NAME_MAX + 1];
+	int command_columns = 0;	/* not including final \0 */
+
+	gethostname(hostname, sizeof(hostname));
 
 	/*
-	 * left justify interval and command, right justify time,
+	 * left justify interval and command, right justify hostname and time,
 	 * clipping all to fit window width
 	 */
 	int hlen = asprintf(&header, _("Every %.1fs: "), interval);
+	int rhlen = asprintf(&right_header, _("%s: %s"), hostname, ts);
 
 	/*
 	 * the rules:
-	 *   width < tsl : print nothing
-	 *   width < tsl + hlen + 1: print ts
-	 *   width = tsl + hlen + 1: print header, ts
-	 *   width < tsl + hlen + 4: print header, ..., ts
-	 *   width < tsl + hlen +    wcommand_columns: print header,
-	 *                           truncated wcommand, ..., ts
-	 *   width > "": print header, wcomand, ts
+	 *   width < rhlen : print nothing
+	 *   width < rhlen + hlen + 1: print hostname, ts
+	 *   width = rhlen + hlen + 1: print header, hostname, ts
+	 *   width < rhlen + hlen + 4: print header, ..., hostname, ts
+	 *   width < rhlen + hlen + wcommand_columns: print header,
+	 *                           truncated wcommand, ..., hostname, ts
+	 *   width > "": print header, wcomand, hostname, ts
 	 * this is slightly different from how it used to be
 	 */
-	if (width < tsl) {
+	if (width < rhlen) {
 		free(header);
+		free(right_header);
 		return;
 	}
-	if (tsl + hlen + 1 <= width) {
+	if (rhlen + hlen + 1 <= width) {
 		mvaddstr(0, 0, header);
-		if (tsl + hlen + 2 <= width) {
-			if (width < tsl + hlen + 4) {
-				mvaddstr(0, width - tsl - 4, "... ");
+		if (rhlen + hlen + 2 <= width) {
+			if (width < rhlen + hlen + 4) {
+				mvaddstr(0, width - rhlen - 4, "... ");
 			} else {
 #ifdef WITH_WATCH8BIT
-				if (width < tsl + hlen + wcommand_columns) {
+	            command_columns = wcswidth(wcommand, -1);
+				if (width < rhlen + hlen + command_columns) {
 					/* print truncated */
-					int available = width - tsl - hlen;
-					int in_use = wcommand_columns;
+					int available = width - rhlen - hlen;
+					int in_use = command_columns;
 					int wcomm_len = wcommand_characters;
 					while (available - 4 < in_use) {
 						wcomm_len--;
 						in_use = wcswidth(wcommand, wcomm_len);
 					}
 					mvaddnwstr(0, hlen, wcommand, wcomm_len);
-					mvaddstr(0, width - tsl - 4, "... ");
+					mvaddstr(0, width - rhlen - 4, "... ");
 				} else {
 					mvaddwstr(0, hlen, wcommand);
 				}
 #else
-				mvaddnstr(0, hlen, command, width - tsl - hlen);
+                command_columns = strlen(command);
+                if (width < rhlen + hlen + command_columns) {
+                    /* print truncated */
+                    mvaddnstr(0, hlen, command, width - rhlen - hlen - 4);
+                    mvaddstr(0, width - rhlen - 4, "... ");
+                } else {
+                    mvaddnstr(0, hlen, command, width - rhlen - hlen);
+                }
 #endif	/* WITH_WATCH8BIT */
 			}
 		}
 	}
-	mvaddstr(0, width - tsl + 1, ts);
+	mvaddstr(0, width - rhlen + 1, right_header);
 	free(header);
+	free(right_header);
 	return;
 }
 
@@ -646,7 +669,6 @@ int main(int argc, char *argv[])
 				 * keeping only */
 #ifdef WITH_WATCH8BIT
 	wchar_t *wcommand = NULL;
-	int wcommand_columns = 0;	/* not including final \0 */
 	int wcommand_characters = 0;	/* not including final \0 */
 #endif	/* WITH_WATCH8BIT */
 
@@ -701,7 +723,7 @@ int main(int argc, char *argv[])
 			flags |= WATCH_EXEC;
 			break;
 		case 'n':
-			interval = strtod_or_err(optarg, _("failed to parse argument"));
+			interval = strtod_nol_or_err(optarg, _("failed to parse argument"));
 			if (interval < 0.1)
 				interval = 0.1;
 			if (interval > UINT_MAX)
@@ -758,7 +780,6 @@ int main(int argc, char *argv[])
 		exit(EXIT_FAILURE);
 	}
 	mbstowcs(wcommand, command, wcommand_characters + 1);
-	wcommand_columns = wcswidth(wcommand, -1);
 #endif	/* WITH_WATCH8BIT */
 
 	get_terminal_size();
@@ -801,7 +822,7 @@ int main(int argc, char *argv[])
 
 		if (show_title)
 #ifdef WITH_WATCH8BIT
-			output_header(wcommand, wcommand_columns, wcommand_characters, interval);
+			output_header(wcommand, wcommand_characters, interval);
 #else
 			output_header(command, interval);
 #endif	/* WITH_WATCH8BIT */
